@@ -208,6 +208,8 @@ lldpd_hardware_cleanup(struct lldpd *cfg, struct lldpd_hardware *hardware)
 	log_debug("alloc", "cleanup hardware port %s", hardware->h_ifname);
 
 	free(hardware->h_lport_previous);
+	free(hardware->h_lchassis_previous_id);
+	free(hardware->h_lport_previous_id);
 	lldpd_port_cleanup(&hardware->h_lport, 1);
 	if (hardware->h_ops && hardware->h_ops->cleanup)
 		hardware->h_ops->cleanup(cfg, hardware);
@@ -904,6 +906,21 @@ lldpd_recv(struct lldpd *cfg, struct lldpd_hardware *hardware, int fd)
 	free(buffer);
 }
 
+static void
+lldpd_send_shutdown(struct lldpd_hardware *hardware)
+{
+	struct lldpd *cfg = hardware->h_cfg;
+	if (cfg->g_config.c_receiveonly || cfg->g_config.c_paused) return;
+	if ((hardware->h_flags & IFF_RUNNING) == 0)
+		return;
+
+	/* It's safe to call `lldp_send_shutdown()` because shutdown LLDPU will
+	 * only be emitted if LLDP was sent on that port. */
+	if (lldp_send_shutdown(hardware->h_cfg, hardware) != 0)
+		log_warnx("send", "unable to send shutdown LLDPDU on %s",
+		    hardware->h_ifname);
+}
+
 void
 lldpd_send(struct lldpd_hardware *hardware)
 {
@@ -1013,7 +1030,7 @@ lldpd_update_localchassis(struct lldpd *cfg)
 		log_debug("localchassis", "use overridden system name `%s`", cfg->g_config.c_hostname);
 		hp = cfg->g_config.c_hostname;
 	} else {
-		if ((hp = priv_gethostbyname()) == NULL)
+		if ((hp = priv_gethostname()) == NULL)
 			fatal("localchassis", "failed to get system name");
 	}
 	free(LOCAL_CHASSIS(cfg)->c_name);
@@ -1118,6 +1135,10 @@ lldpd_exit(struct lldpd *cfg)
 {
 	struct lldpd_hardware *hardware, *hardware_next;
 	log_debug("main", "exit lldpd");
+
+	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries)
+		lldpd_send_shutdown(hardware);
+
 	close(cfg->g_ctl);
 	priv_ctl_cleanup(cfg->g_ctlname);
 	log_debug("main", "cleanup hardware information");
@@ -1314,11 +1335,13 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	int receiveonly = 0;
 	int ctl;
 
+#ifdef ENABLE_PRIVSEP
 	/* Non privileged user */
 	struct passwd *user;
 	struct group *group;
 	uid_t uid;
 	gid_t gid;
+#endif
 
 	saved_argv = argv;
 
@@ -1472,12 +1495,14 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	log_debug("main", "lldpd starting...");
 
 	/* Grab uid and gid to use for priv sep */
+#ifdef ENABLE_PRIVSEP
 	if ((user = getpwnam(PRIVSEP_USER)) == NULL)
 		fatal("main", "no " PRIVSEP_USER " user for privilege separation");
 	uid = user->pw_uid;
 	if ((group = getgrnam(PRIVSEP_GROUP)) == NULL)
 		fatal("main", "no " PRIVSEP_GROUP " group for privilege separation");
 	gid = group->gr_gid;
+#endif
 
 	/* Create and setup socket */
 	int retry = 1;
@@ -1492,7 +1517,7 @@ lldpd_main(int argc, char *argv[], char *envp[])
 				/* Another instance is running */
 				close(tfd);
 				log_warnx("main", "another instance is running, please stop it");
-				fatalx("giving up");
+				fatalx("main", "giving up");
 			} else if (errno == ECONNREFUSED) {
 				/* Nobody is listening */
 				log_info("main", "old control socket is present, clean it");
@@ -1500,17 +1525,19 @@ lldpd_main(int argc, char *argv[], char *envp[])
 				continue;
 			}
 			log_warn("main", "cannot determine if another daemon is already running");
-			fatalx("giving up");
+			fatalx("main", "giving up");
 		}
 		log_warn("main", "unable to create control socket");
-		fatalx("giving up");
+		fatalx("main", "giving up");
 	}
+#ifdef ENABLE_PRIVSEP
 	if (chown(ctlname, uid, gid) == -1)
 		log_warn("main", "unable to chown control socket");
 	if (chmod(ctlname,
 		S_IRUSR | S_IWUSR | S_IXUSR |
 		S_IRGRP | S_IWGRP | S_IXGRP) == -1)
 		log_warn("main", "unable to chmod control socket");
+#endif
 
 	/* Disable SIGPIPE */
 	signal(SIGPIPE, SIG_IGN);
@@ -1555,7 +1582,11 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	}
 
 	log_debug("main", "initialize privilege separation");
+#ifdef ENABLE_PRIVSEP
 	priv_init(PRIVSEP_CHROOT, ctl, uid, gid);
+#else
+	priv_init(PRIVSEP_CHROOT, ctl, 0, 0);
+#endif
 
 	/* Initialization of global configuration */
 	if ((cfg = (struct lldpd *)
